function [speedmatrix,speedindex,speedindex2,speedindexA,rowdata,stimindexA,stimindexB,...
    pWelch_pre,pWelch_stim,pWelch_post,fWelch,...
    amplT,phaseT,amplB,phaseB,amplS,phaseS,amplB2,phaseB2,eegFilt,lagDiff,THDr,animalIndex] = andor_INCfreemov_STEP5_f1(stim,arena,freq,select,pretime,stimtime)

WelchWindow = 1000;
WelchNoverlap = 500;
WelchNfft = 2048;

% get pyr channel numbers:
pyrCh(:,1) = {'1868'          ,'1943'       ,'1957'       ,'1985'       ,'2017'          ,'2022A','2022B','2026' ,'2027','2040','2045','2046'};
pyrCh(:,2) = { 4              , 12          , 20          , 7           , 9              , 6     , 23    , 6     , 4    , 11   , 1    , 4    };

% radCh(:,1) = {'1868'          ,'1943'       ,'1957'       ,'1985'       ,'2017'          ,'2022A','2022B','2026' ,'2027','2040','2045','2046'};
% radCh(:,2) = {[5,6,7,8]       ,[14,15]      ,[21,22,23,24],[9,12]       ,[12,14]         ,[]     ,[]     ,[7,8]  ,      ,      ,[3,4] ,      };
% 
% lamCh(:,1) = {'1868'          ,'1943'       ,'1957'       ,'1985'       ,'2017'          ,'2022A','2022B','2026' ,'2027','2040','2045','2046'};
% lamCh(:,2) = {[11,12]         ,[17,18,19,20],[25,26,27,28],[14,15,16,17],[16,17,18,19,20],[]     ,[]     ,[9,10] ,      ,      ,[8,9] ,      };
% 
% molCh(:,1) = {'1868'          ,'1943'       ,'1957'       ,'1985'       ,'2017'          ,'2022A','2022B','2026' ,'2027','2040','2045','2046'};
% molCh(:,2) = {[13,14,15]      ,[22]         ,[31]         ,[19,21]      ,[21,22]         ,[]     ,[]     ,[11,12],      ,      ,      ,      };

% hilCh(:,1) = {'1868'          ,'1943'       ,'1957'       ,'1985'       ,'2017'          ,'2022A','2022B','2026' ,'2027','2040','2045','2046'};
% hilCh(:,2) = {[16,17,18,19,20],[27,28]      ,[]           ,[]           ,[24,25,26]      ,       ,       ,[13,14],      ,      ,[]    ,      };
% 
% badCh(:,1) = {'1868'          ,'1943'       ,'1957'       ,'1985'       ,'2017'          ,'2022A','2022B','2026' ,'2027','2040','2045','2046'};
% badCh(:,2) = {                ,             ,             ,             ,[]              ,       ,       ,[]     ,      ,      ,      , 8    };


% load data
filename = ['INC_res2_' stim '_' arena '_' freq '.mat'];
load(filename);

% remove NAN rows
if ~strcmp(arena,'urethane')
    toremove = false(size(csvIndex,1),1);
    for k = 1:size(csvIndex,1)
        if isempty(speed{k,1})
            toremove(k) = true;
        else
            if all(isnan(speed{k,1}(csvIndex(k,3):csvIndex(k,4))))
                toremove(k) = true;
            end
        end
    end
    speed = speed(~toremove,:);
    speed2 = speed2(~toremove,:);
    pulseIndex = pulseIndex(~toremove,:);
    eeg = eeg(~toremove,:);
    rowInfo = rowInfo(~toremove,:);
    csvIndex = csvIndex(~toremove,:);
    stimindex = stimIndex(~toremove,:);
end

% select good files
selectindex = false(size(rowInfo,1),1);
for k = 1:length(selectindex)
    temp = NaN(1,numel(select));
    for xx = 1:numel(select)
        a = strfind(rowInfo{k,1},select{xx});
        if ~isempty(a); temp(xx) = a; end
    end
    if any(~isnan(temp)); selectindex(k) = true; end
end


% remove noisy (above No 28) and bad (2ms stim length: files No 14-22) files of 1868
if ismember('1868',select)
    for r = 1:size(rowInfo,1)
        if isequal('1868',rowInfo{r,1}(1:4))
            temp = str2num(rowInfo{r,1}(6:end));
            if temp > 28
                selectindex(r) = false;
            elseif and(temp >= 14, temp <= 22)
                selectindex(r) = false;
            end
        end
    end
end
    

if ~all(~selectindex)
    if ~strcmp(arena,'urethane')
        speed = speed(selectindex,:);
        speed2 = speed2(selectindex,:);
        csvIndex = csvIndex(selectindex,:);
    else
        speed = [];
        speed2 = [];
        csvIndex = [];
    end
    pulseIndex = pulseIndex(selectindex,:);
    eeg = eeg(selectindex,:);
    rowInfo = rowInfo(selectindex,:);
    
    stimindex = stimindex(selectindex,:);
    
    % select data only from one animal
    for nn = 1:numel(select)
        T = cellfun(@(x) ~isempty(strfind(x,select{nn})),rowInfo(:,1));
        animalIndex{nn,1} = find(T);
    end
    
    rowdata = rowInfo;
    if ismember('2022',select)
        fileHelp = cellfun(@(x) str2num(x(6:end)),rowInfo(:,1));
    end
    
    if ~strcmp(arena,'urethane')
        ind2max = max(csvIndex(:,2));
    end
    ind2maxEEG = max(stimindex(:,2));
    csvIndex2 = [];
    stimIndex2 = [];
    for k = 1:size(stimindex,1)
        % speed
        if ~strcmp(arena,'urethane')
            if csvIndex(k,2)<ind2max
                csvIndex2(k,:) = csvIndex(k,:)+(ind2max-csvIndex(k,2));
            else
                csvIndex2(k,:) = csvIndex(k,:);
            end
        end
        
        % eeg
        if stimindex(k,2)<ind2maxEEG
            stimIndex2(k,:) = stimindex(k,:)+(ind2maxEEG-stimindex(k,2));
        else
            stimIndex2(k,:) = stimindex(k,:);
        end
    end
    
    % create matrix
    if ~strcmp(arena,'urethane')
        speedmatrix = NaN(size(csvIndex2,1),max(csvIndex2(:,end)));
    else
        speedmatrix = [];
    end
    
    eegmatrix = NaN(size(stimIndex2,1),max(stimIndex2(:,end)));
    
    pyrN = NaN(size(stimIndex2,1),1);
    amplT = NaN(size(stimIndex2,1),pretime+stimtime+1);
    phaseT = NaN(size(stimIndex2,1),pretime+stimtime+1);
    amplB = NaN(size(stimIndex2,1),pretime+stimtime+1);
    phaseB = NaN(size(stimIndex2,1),pretime+stimtime+1);
    amplS = NaN(size(stimIndex2,1),pretime+stimtime+1);
    phaseS = NaN(size(stimIndex2,1),pretime+stimtime+1);
    amplB2 = NaN(size(stimIndex2,1),pretime+stimtime+1);
    phaseB2 = NaN(size(stimIndex2,1),pretime+stimtime+1);
    eegFilt = NaN(size(stimIndex2,1),pretime+stimtime+1);
    for k =1:size(stimIndex2,1)
        if ~strcmp(arena,'urethane')
            % speed
            temp = speed{k,1};
            if length(temp)>csvIndex2(k,end)-csvIndex2(k,1)
                temp = temp(1:(csvIndex2(k,end)-csvIndex2(k,1)));
            end
            speedmatrix(k,csvIndex2(k,1):csvIndex2(k,end)-1) = temp;
        end
        
        % eeg
        tempeeg = eeg{k,1};
        temprowdata = rowdata{k,1}(1:4);
        if isequal(temprowdata,'2022')
            if fileHelp(k,1) < 25
                temprowdata = [temprowdata 'A'];
            elseif fileHelp(k,1) >= 25
                temprowdata = [temprowdata 'B'];
            end
        end
        pyrN(k,1) = pyrCh{ismember(pyrCh(:,1),temprowdata),2};
        tempeegPyr = tempeeg(:,pyrN(k,1));
        
        bvar = fir1(2048, 1/(1000/2),'high'); % filter the eeg <1Hz
        tempeegPyr = filtfilt(bvar,1,tempeegPyr);
        
        if length(tempeegPyr)>stimIndex2(k,end)-stimIndex2(k,1)+1
            tempeegPyr = tempeegPyr(1:(stimIndex2(k,end)-stimIndex2(k,1))+1);
            tempeeg = tempeeg(1:(stimIndex2(k,end)-stimIndex2(k,1))+1,:);
        end
        eegmatrix(k,stimIndex2(k,1):stimIndex2(k,end)) = tempeegPyr;
        
        if stimindex(k,3)-pretime > 0
            if stimindex(k,3)+stimtime<= length(tempeegPyr)
                %eeg, hilbert ampl, phase average
                %5-12 Hz
                
                bvar = fir1(2048,[5/(1000/2) 12/(1000/2)]);
                eegF = filtfilt(bvar,1,tempeegPyr);
                
                eegH = hilbert(eegF);
                ampl = abs(eegH);
                phase = radtodeg(angle(eegH));
                amplT(k,:) = ampl(stimindex(k,3)-pretime:stimindex(k,3)+stimtime);
                phaseT(k,:) = phase(stimindex(k,3)-pretime:stimindex(k,3)+stimtime);
                
                
                %23-36 Hz
                bvar = fir1(2048,[23/(1000/2) 36/(1000/2)]);
                eegF = filtfilt(bvar,1,tempeegPyr);
                eegH = hilbert(eegF);
                ampl = abs(eegH);
                phase = radtodeg(angle(eegH));
                amplB(k,:) = ampl(stimindex(k,3)-pretime:stimindex(k,3)+stimtime);
                phaseB(k,:) = phase(stimindex(k,3)-pretime:stimindex(k,3)+stimtime);
                
                %23-27 Hz
                bvar = fir1(2048,[23/(1000/2) 27/(1000/2)]);
                eegF = filtfilt(bvar,1,tempeegPyr);
                eegH = hilbert(eegF);
                ampl = abs(eegH);
                phase = radtodeg(angle(eegH));
                amplS(k,:) = ampl(stimindex(k,3)-pretime:stimindex(k,3)+stimtime);
                phaseS(k,:) = phase(stimindex(k,3)-pretime:stimindex(k,3)+stimtime);
                
                %29-36 Hz
                bvar = fir1(2048,[29/(1000/2) 36/(1000/2)]);
                eegF = filtfilt(bvar,1,tempeegPyr);
                eegH = hilbert(eegF);
                ampl = abs(eegH);
                phase = radtodeg(angle(eegH));
                amplB2(k,:) = ampl(stimindex(k,3)-pretime:stimindex(k,3)+stimtime);
                phaseB2(k,:) = phase(stimindex(k,3)-pretime:stimindex(k,3)+stimtime);
                
                
                bvar = fir1(2048,[1/(1000/2) 300/(1000/2)]);
                eegF = filtfilt(bvar,1,tempeegPyr);
                eegFilt(k,:) = eegF(stimindex(k,3)-pretime:stimindex(k,3)+stimtime);
                
                [pWelch_pre(k,:), fWelch_pre(k,:)] = pwelch(eegF(stimindex(k,3)-pretime-1:stimindex(k,3)-1),WelchWindow,WelchNoverlap,WelchNfft,1000);
                [pWelch_stim(k,:), fWelch_stim(k,:)] = pwelch(eegF(stimindex(k,3):stimindex(k,4)),WelchWindow,WelchNoverlap,WelchNfft,1000);
                [pWelch_post(k,:), fWelch_post(k,:)] = pwelch(eegF(stimindex(k,4)+1:stimindex(k,4)+pretime+1),WelchWindow,WelchNoverlap,WelchNfft,1000);
                
                bvar = fir1(2048,300/(1000/2),'high');
                eegForREM = filtfilt(bvar,1,tempeeg);
                
                
                % REM index
                
                step = 25; % ms = datapoints
                stepNo = fix(size(eegForREM,1)/step);
                for ee = 1:size(eegForREM,2)
                    for eee = 1:stepNo
                        [acor, lag] = xcorr(eegForREM(((eee-1)*step+1):(eee*step),ee),eegForREM(((eee-1)*step+1):(eee*step),pyrN(k,1)));
                        [~,tempI] = max(abs(acor));
                        templag = lag(tempI);
                        lagDiff{k}(eee,ee) = max(templag);
                    end
                end
                            
                [fr, ~, power, scale] = wavecall_andor(reshape(tempeegPyr,1,[]), 1000, 'no');
                power = bsxfun(@rdivide, power, scale');
                th1 = knnsearch(fr',5);
                th2 = knnsearch(fr',12);
                d1 = knnsearch(fr',1);
                d2 = knnsearch(fr',4);
                powerTH = power(th2:th1,:);
                powerD = power(d2:d1,:);
                powerTH = sumskipnan(powerTH);
                powerD = sumskipnan(powerD);
                THDr{k,1} = powerTH./powerD;
            end 
        end
    end
    
    fWelch = fWelch_pre; 
    
    if ~strcmp(arena,'urethane')
        speedindex = csvIndex2(:,3);
        speedindex2 = csvIndex2(:,4);
        
        speedindexA = csvIndex(:,4);
    else
        speedindex = [];
        speedindex2 = [];
        
        speedindexA = [];
    end
    
    
    stimindexA = stimindex(:,3)-pretime+1;
    stimindexB = stimindex(:,4)-pretime+1;
  

else
    speedmatrix = [];
    eegmatrix = [];
    speedindex = [];
    speedindex2 = [];
    eegindex = [];
    eegindex2 = [];
    rowdata = [];
    
    pulseIndex = [];
    eeg = [];
    pyrN = [];
    stimindexA = [];
    stimindexB = [];
    
end